@@ -7,6 +7,10 @@ from _collections import OrderedDict
 from . import HTMLPlotter
 import os
 import sys
+import xarray as xr
+
+# xarray options
+xr.set_options(keep_attrs=True)
 
 # runs without Qt
 if not 'PYTPLOT_NO_GRAPHICS' in os.environ:
